<?php

declare(strict_types=1);

namespace FiloBlu\RuleDayFromBirthday\Model\SalesRule;

use Exception;
use FiloBlu\RuleDayFromBirthday\Api\CustomCouponRepositoryInterface;
use FiloBlu\RuleDayFromBirthday\Api\Data\CustomCouponInterface;
use Magento\Framework\EntityManager\MetadataPool;
use Magento\Framework\EntityManager\Operation\ExtensionInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\SalesRule\Api\Data\RuleInterface;

/**
 * @class ReadHandler
 * @package FiloBlu\RuleDayFromBirthday\Model
 */
class ReadHandler implements ExtensionInterface
{

    /**
     * @var MetadataPool
     */
    protected $metadataPool;
    /**
     * @var CustomCouponRepositoryInterface
     */
    private $customCouponRepository;

    /**
     * @param MetadataPool $metadataPool
     * @param CustomCouponRepositoryInterface $customCouponRepository
     */
    public function __construct(
        MetadataPool $metadataPool,
        CustomCouponRepositoryInterface $customCouponRepository
    ) {
        $this->metadataPool = $metadataPool;
        $this->customCouponRepository = $customCouponRepository;
    }

    /**
     * @param $entity
     * @param array $arguments
     * @return bool|object
     * @throws Exception
     */
    public function execute($entity, $arguments = [])
    {
        $attributes = $entity->getExtensionAttributes() ?: [];
        $metadata = $this->metadataPool->getMetadata(RuleInterface::class);
        $rowId = $entity->getData($metadata->getLinkField());

        if ($rowId) {
            try {
                /** @var CustomCouponInterface $customCoupon */
                $customCoupon = $this->customCouponRepository->getById($rowId);
                $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::RULE_ID] = $customCoupon->getRuleId();
                $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::EXPIRATION_DAYS] = $customCoupon->getExpirationDays();
                $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::TYPE] = $customCoupon->getType();
                $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::PREFIX] = $customCoupon->getPrefix();
                $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::SUFFIX] = $customCoupon->getSuffix();
                $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::LENGTH] = $customCoupon->getLength();
                $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::FORMAT] = $customCoupon->getFormat();
                $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::DASH] = $customCoupon->getDash();
                $attributes['filoblu_sales_rule_coupon']['row_id'] = $rowId;
            } catch (NoSuchEntityException $exception) {
                $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::RULE_ID] = 0;
                $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::EXPIRATION_DAYS] = 0;
                $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::TYPE] = '';
                $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::PREFIX] = '';
                $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::SUFFIX] = '';
                $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::LENGTH] = 0;
                $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::FORMAT] = '';
                $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::DASH] = 0;
                $attributes['filoblu_sales_rule_coupon']['row_id'] = $rowId;
            }
        }

        $entity->setExtensionAttributes($attributes);

        if ($entity->getData('filoblu_coupon_expire_date') === null) {
            $entity->setData('filoblu_coupon_expire_date', $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::EXPIRATION_DAYS]);
        }

        if ($entity->getData('filoblu_birthday_coupon') === null) {
            $entity->setData('filoblu_birthday_coupon', $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::TYPE]);
        }

        if ($entity->getData('coupons_length') === null) {
            $entity->setData('coupons_length', $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::LENGTH]);
        }

        if ($entity->getData('coupons_format') === null) {
            $entity->setData('coupons_format', $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::FORMAT]);
        }

        if ($entity->getData('coupons_prefix') === null) {
            $entity->setData('coupons_prefix', $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::PREFIX]);
        }

        if ($entity->getData('coupons_suffix') === null) {
            $entity->setData('coupons_suffix', $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::SUFFIX]);
        }

        if ($entity->getData('coupons_dash') === null) {
            $entity->setData('coupons_dash', $attributes['filoblu_sales_rule_coupon'][CustomCouponInterface::DASH]);
        }

        return $entity;
    }
}
